import 'package:dio/dio.dart';
import 'package:project_core/core/exception/AppException.dart';
import 'package:project_core/core/http/AppHttpResponse.dart';
import 'package:project_core/core/model/Entity.dart';
import 'package:project_core/core/provider/RestProvider.dart';
import 'package:project_core/core/provider/interceptor/AuthTokenInterceptor.dart';
import 'package:project_core/env/EnvConfig.dart';

class DioRestProvider<T extends Entity> extends RestProvider<T> {
  Dio client = Dio();
  bool withTokenInterceptor = false;

  DioRestProvider(String apiUrl) : super(apiUrl) {
    withTokenInterceptor = false;
    client.interceptors.add(TokenInterceptor());
    client.interceptors.add(LogInterceptor());
    client.options.baseUrl = Env.instance.config.apiUrl;
    client.options.connectTimeout = 50000;
  }

  DioRestProvider.tokenInterceptor(String apiUrl) : super(apiUrl) {
    withTokenInterceptor = true;
    client.interceptors.add(TokenInterceptor());
    client.interceptors.add(LogInterceptor());
    client.options.baseUrl = Env.instance.config.apiUrl;
    client.options.connectTimeout = 50000;
  }

  @override
  AppHttpResponse createHttpResponse(requestResponse) {
    if (requestResponse is Response) {
      return makeFromDioResponse(requestResponse);
    } else if (requestResponse is DioError) {
      if (DioErrorType.RECEIVE_TIMEOUT == requestResponse.type ||
          DioErrorType.CONNECT_TIMEOUT == requestResponse.type) {
        throw FetchDataException(
            "O servidor não está acessível. Verifique sua conexão com a internet e tente novamente");
      } else if (DioErrorType.RESPONSE == requestResponse.type) {
        return makeFromDioResponse(requestResponse.response);
      } else if (DioErrorType.DEFAULT == requestResponse.type) {
        if (requestResponse.message.contains('SocketException')) {
          throw FetchDataException(
              "O servidor não está acessível. Por favor, contate com o suporte.");
        }
      } else {
        throw FetchDataException(
            "Problema ao conectar-se ao servidor. Por favor, tente novamente.");
      }

      /*if (requestResponse.response != null) {
        return makeFromDioResponse(requestResponse.response);
      } else {
        return AppHttpResponse(code: 0, body: requestResponse.message);
      }*/
    }
    return AppHttpResponse(code: 0, body: "Tivemos um problema não esperado ao realizar a sua requisição. Por favor, contate com o suporte.");
  }

  AppHttpResponse makeFromDioResponse(Response response) {
    return AppHttpResponse(code: response.statusCode, body: response.data);
  }

  getTokenClient() {
    client.options.extra["token"] = true;
    return client;
  }

  @override
  getClient() {
    client.options.extra["token"] = withTokenInterceptor;
    return client;
  }
}
